using System;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;

namespace SdiBaseSimple
{
    /// <summary>
    /// Application context for Sdi application.<p>
    /// <b>Application context for Sdi application.</b><br>
    /// Call CreateDocument from main form of application.<br>
    /// Call SetView for each view to be added to main form.<br>
    /// The doc can be accessed through this context.<br>
    /// The active view can be accessed through this context.<br>
	/// </summary>
	public class SdiApplicationContext : ApplicationContext
	{
		protected SdiDoc sdiDoc;
        protected Dictionary<Control, SdiView> hashActive = new Dictionary<Control, SdiView>();
        protected List<SdiMainForm> listMainForm = new List<SdiMainForm>();

        /// <summary>
        /// Default Constructor.
        /// <p>
        /// The default constructor does not do anything, but it is safer to
        /// always create a default constructor: the designer calls them at
        /// the strangest times.
        /// </summary>
		public SdiApplicationContext()
		{
		}

        /// <summary>
        ///    Add a main form to the current context.
        /// <p>
        /// This should be called from the constructor of the main form class.
        /// </summary>
        /// <param name="mainForm">The main form to add.</param>
        public void AddMainForm(SdiMainForm mainForm)
        {
            listMainForm.Add(mainForm);
            Console.WriteLine("Main Form: " + listMainForm.Count.ToString());
            mainForm.FormClosed += new FormClosedEventHandler(mainForm_FormClosed);
        }

        /// <summary>
        ///    Create a main form and add it to the context.
        /// <p>
        /// Call this method when a main form that is already open needs to 
        /// create another top level form.
        /// </summary>
        /// <param name="type">The type of the main form.</param>
        /// <returns>The main form that was created.</returns>
        public SdiMainForm CreateMainForm(Type type)
        {
            Object[] args = new Object[] { this };
            SdiMainForm mainForm = (SdiMainForm)Activator.CreateInstance(type, args);
            return mainForm;
        }

        protected void mainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            SdiMainForm mainForm = (SdiMainForm)sender;
            listMainForm.Remove(mainForm);
            if (listMainForm.Count > 0 && this.MainForm == mainForm)
            {
                this.MainForm = listMainForm[0];
            }
        }

        /// <summary>
        /// Calls constructor of type, using reflection.
        /// <p>
        /// If a view has not been created, then it can be created in the
        /// base class using reflection. This saves the programmer from 
        /// having to construct each view before adding it to a container.
        /// </summary>
        /// <param name="name">The name to be used to retrieve this view.</param>
        /// <param name="type">The type of the class that for this view.</param>
        /// <returns>Returns the view that is created as SdiView.</returns>
        protected SdiView CreateView(String name, Type type)
        {
            Object[] args = new Object[] { name, this };
            SdiView viewToCreate = (SdiView)Activator.CreateInstance(type, args);
            return viewToCreate;
        }

        #region Public members to be called from main form

        /// <summary>
        /// Call this method from the main form of an Sdi application to create the 
        /// document for the application.
        /// </summary>
        /// <param name="type">The type of the document class.</param>
        /// <returns>The document that was created, as SdiDoc.</returns>
		public SdiDoc CreateDocument(Type type)
		{
			Object[] args = new Object[]{this};
			Doc = (SdiDoc) Activator.CreateInstance (type, args);
			return Doc;
		}

        /// <summary>
        /// Make a view active in a container.
        /// <p>
        /// This will construct the view if it has not already been constructed.<br>
        /// Call it for each view that will be displayed.<br>
        /// This method will make this view active and hide the previous view.
        /// </summary>
        /// <param name="strViewName">The name used to retrieve the view.</param>
        /// <param name="type">The type of the view class.</param>
        /// <param name="control">The container control that the view should be added to.</param>
        /// <returns>The view that was activated; returned as SdiView.</returns>
		public SdiView SetView(String strViewName, Type type, Control control) 
		{
			SdiBaseSimple.SdiView viewToSet = GetView(strViewName, type, control);
			if (viewToSet == null) 
			{
				viewToSet = CreateView(strViewName, type);
			}
			if (!control.Controls.Contains(viewToSet)) 
			{
				control.Controls.Add(viewToSet);
			}
			SdiView active = GetActiveView(control);
			if (active != null) active.Hide();
			hashActive[control] = viewToSet;
			viewToSet.Doc_Update(null);
			viewToSet.BringToFront();
			viewToSet.Show();
			return viewToSet;
        }

        /// <summary>
        /// Retrieve a view by name.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="key">The name of the view to retrieve.</param>
        /// <param name="type">The type of the view to retrieve.</param>
        /// <param name="container">The container that contains the view.</param>
        /// <returns>The view; returned as SdiView.</returns>
        public SdiView GetView(String key, Type type, Control container)
        {
            foreach (Control control in container.Controls)
            {
                SdiView view = control as SdiView;
                if (view != null && view.Name == key)
                {
                    return view;
                }
            }
            return null;
        }

        /// <summary>
        /// Returns the current view that is active in a container in the main form.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="container">The container that contains the active view.</param>
        /// <returns>The view; returned as SdiView.</returns>
		public SdiView GetActiveView(Control container) 
		{
            try
            {
                return hashActive[container];
            }
            catch (KeyNotFoundException)
            {
                return null;
            } 
	    }

        /// <summary>
        /// Property that returns the document, as SdiDoc.
        /// </summary>
		public SdiDoc Doc
		{
			get
			{
				return sdiDoc;
			}
			set
			{
				sdiDoc = (SdiDoc) value; 
			}
        }

        /// <summary>
        /// Property that returns the list of top level forms.
        /// </summary>
        public List<SdiMainForm> TopLevelForms
        {
            get { return listMainForm; }
        }

        #endregion

    }
}
