//Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

//This source code is intended only as a supplement to Microsoft
//Development Tools and/or on-line documentation.  See these other
//materials for detailed information regarding Microsoft code samples.

//THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.

namespace SdiBaseSimple
{
	using System;
	using System.IO ;
	using System.Drawing;
	using System.Runtime.Serialization;
	using System.Runtime.Serialization.Formatters.Binary ;
	using System.Windows.Forms ;

	/// <summary>
    /// Base class for document in Sdi application.
    /// <p>
    /// Base class for document in Sdi application.<br>
    /// There will be one doc for entire app.<br>
    /// Whenever the doc changes, call UpdateAllViews.<br>
    /// Override DeleteContents to reset extended class to initial state.<br>
    /// Set Modified whenever the doc changes. There is no built in
    ///   functionality for Modified. All use of Modified must be implemented
    ///   in extended class.
	/// </summary>
	[SerializableAttribute]
	public class SdiDoc
	{
        /// <summary>
        /// Each view is subscribed to this event.
        /// <p>
        /// As part of the Sdi framework, each view will subscribe to this event
        /// when the Application:SetView method is called.
        /// </summary>
        public event UpdateEvent Update;

        private bool m_bIsDirty;
		private SdiApplicationContext m_context;
        public int countDoc;
        static int count = 0;
		
        /// <summary>
        /// Each derived class <b>MUST</b> create a constructor with this signature
        /// and call the base class constructor.
        /// <pre>
        /// public ExtendedDoc(SdiApplicationContext context) : base(context) {}
        /// </pre>
        /// </summary>
        /// <param name="context">Pass the application context from the main form to this method.</param>
        public SdiDoc(SdiApplicationContext context)
		{
			m_bIsDirty= false;		
			m_context = context;
            countDoc = ++count;
        }

        #region Methods to be overridden in the extended doc class

        /// <summary>
        /// Override this method in the extended doc to reset content.
        /// <p>
        /// Each derived class will have its own data. Override this method
        /// to reset the data to its initial state.
        /// <p>
        /// Be sure to call the base class UpdateAllViews in the overridden
        /// method or call it directly in the overridden method.
        /// </summary>
        public virtual void DeleteContents()
		{
		    UpdateAllViews(null);
        }

        #endregion

        #region Methods and Properties that can be access from extended doc class

        /// <summary>
        /// Call this method everytime the document changes.
        /// <p>
        /// This method will fire the Update event. All views that have been
        /// created with SdiApplicationContext.SetView will be notified of this event.
        /// </summary>
        /// <param name="sender">If the sender is not null, then it should be
        /// the current, active view. The Update handler in that view can test
        /// the parameter to see if it is the view that initiated the change.
        /// This is useful if the view already updated itself, before the 
        /// document was notified of the change.
        /// </param>
        public void UpdateAllViews(Object sender) 
		{
			//Update routine can test if sender is same as view.
			if (Update != null) Update(sender);
		}


        /// <summary>
        /// A property that can be used to keep track if the document has changed.
        /// <p>
        /// There is no built in functionality for this property. It is up to the
        /// user to set this whenever the document changes and to clear it when
        /// the document is new or has been saved.
        /// </summary>
		public bool Modified 
		{
			get 
			{
				return m_bIsDirty;
			}
			set
			{
				m_bIsDirty = value;
			}
		}

        /// <summary>
        /// Return the context for this application.
        /// <p>
        /// The context was created in the main form and sent to the doc when the
        /// doc was created.
        /// </summary>
		public SdiApplicationContext Context 
		{
			get 
			{
				return m_context;
			}
        }

        #endregion
    }

    /// <summary>
    /// Delegate that is used to send the Update message.
    /// </summary>
    /// <param name="sender"></param>
	public delegate void UpdateEvent(Object sender);
}
