//Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

//This source code is intended only as a supplement to Microsoft
//Development Tools and/or on-line documentation.  See these other
//materials for detailed information regarding Microsoft code samples.

//THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.
using System.Diagnostics;

namespace SdiBaseSimple
{
	using System;
	using System.IO ;
	using System.Drawing;
	using System.Drawing.Printing;
	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;

	/// <summary>
	/// SdiView is a user control. 
    /// All views should extend from this class. 
	/// </summary>
	public class SdiView : UserControl
	{
        private SdiApplicationContext m_context;

        /// <summary>
        ///    Default Constructor.
        /// </summary>
		public SdiView() 
		{
            SdiInitializeComponent();
		}

        //
        //
        /// <summary>
        /// Create a constructor in all views with this signature.
        /// <p>
        /// The extended constructor <b>MUST</b> call the base contstructor.
        /// <pre>
        /// public ExtendedView(String name, SdiApplicationContext context) 
        ///     : base(name, context)
        /// {
        /// 	...
        /// }
        /// </pre>
        /// If you modify the extended view in the designer, be sure to
        /// call InitializeComponent in the extended view from the view's constructor.
        /// InitializeComponent will not be created until you modify the extended
        /// view in the designer.
        /// <pre>
        /// public ExtendedView(String name, SdiApplicationContext context) 
        ///     : base(name, context)
        /// {
        /// 	...
        ///     InitializeComponent();
        ///     ...
        /// }
        /// </pre>
        /// </summary>
        /// <param name="name">The name to be used to retrieve the view</param>
        /// <param name="context">The application context from the main form</param>
		public SdiView(String name, SdiApplicationContext context) : this()
		{
			m_context = context;
			Name = name;
			m_context.Doc.Update += new UpdateEvent(Doc_Update);
			this.Paint += new PaintEventHandler(SdiView_Paint);
			Hide();
        }

        protected override void Dispose(bool disposing)
        {
            m_context.Doc.Update -= Doc_Update;
            Paint -= SdiView_Paint;
            base.Dispose(disposing);
        }


        #region Methods to override in the extended class

        /// <summary>
        ///    Override in derived class for actions to perform when doc changes.
        /// <p>
        /// Be sure to call the base class Doc_Update or call Invalidate(true) in
        /// the overriden method.
        /// </summary>
        /// <param name="sender">The view that initiated the change to the doc.</param>
		public virtual void Doc_Update(Object sender)
		{
			Invalidate(true);
		}

        /// <summary>
        ///    Override this method in the extended class.
        /// <p>
        /// Each view that is created with SdiApplication.SetView is subscribed
        /// to the paint event. Override this method in the extended class to
        /// handled painting. It is not necessary to subscribe to the paint handler
        /// in the extended class.
        /// </summary>
        /// <param name="sender">Object that initiated paint.</param>
        /// <param name="e">Normal event args for paint</param>
		public virtual void SdiView_Paint(object sender, PaintEventArgs e)
		{
			//override in extended class for polymorphic painting
		}

        #endregion

        /// <summary>
        ///    Read-only property to access the application context.
        /// </summary>
        public SdiApplicationContext Context
        {
            get
            {
                return m_context;
            }
        }

        /// <summary>
        ///    Default appearance of view.
        /// <p>
        /// This is the default appearance of view before any changes are made
        /// in the extended class. This method should not be changed or
        /// overridden. Make all changes in extended view class. Any changes
        /// made to the extended class in the designer will be reflected in IntializeComponent in the 
        /// extended class.
        /// </summary>
        protected void SdiInitializeComponent()
        {
            this.SuspendLayout();
            // 
            // SdiView
            // 
            this.BackColor = System.Drawing.SystemColors.ActiveCaption;
            this.Name = "SdiView";
            this.Size = new System.Drawing.Size(266, 287);
            this.ResumeLayout(false);

        }
	}
}
